package gregapi.tileentity.base;

import static gregapi.data.CS.ERR;
import static gregapi.data.CS.F;
import static gregapi.data.CS.NW_API;
import static gregapi.data.CS.T;
import gregapi.network.INetworkHandler;
import gregapi.network.IPacket;
import gregapi.tileentity.ITileEntitySynchronising;
import net.minecraft.entity.player.EntityPlayerMP;

/**
 * @author Gregorius Techneticies
 * 
 * TileEntity with Network Code
 */
public abstract class TileEntityBase03TicksAndSync extends TileEntityBase02AdjacentTEBuffer implements ITileEntitySynchronising {
	/** Variable for seeing if the Tick Function is called right now. */
	public boolean mIsRunningTick = F;
	
	/** Variable for updating Data to the Client */
	private boolean mSendClientData = F;
	
	/** @return a Packet containing all Data which has to be synchronised to the Client */
	public abstract IPacket getClientDataPacket(boolean aSendAll);
	
	/** sends all Data to the Clients in Range */
	public void sendClientData(boolean aSendAll, EntityPlayerMP aPlayer) {
		if (aPlayer == null) {
			getNetworkHandler().sendToAllPlayersInRange(getClientDataPacket(aSendAll), worldObj, getCoords());
		} else {
			// If it is about to send its Data to all Players in Range, then this individual update is not needed.
			if (!mSendClientData) getNetworkHandler().sendToPlayer(getClientDataPacket(aSendAll), aPlayer);
		}
	}
	
	/** @return the used Network Handler. Defaults to the API Handler. */
	public INetworkHandler getNetworkHandler() {return NW_API;}
	
	/** Called to send all Data to the close Clients */
	public void updateClientData() {mSendClientData = T;}
	
	@Override public void onCoordinateChange() {super.onCoordinateChange(); updateClientData();}
	
	@Override public final net.minecraft.network.Packet getDescriptionPacket() {return null;}
	
    @Override
    public void validate() {
    	super.validate();
    	updateClientData();
    }
    
	@Override
    public final void sendUpdateToPlayer(EntityPlayerMP aPlayer) {
		sendClientData(T, aPlayer);
    }
    
    @Override
    public final void updateEntity() {
    	mIsRunningTick = T;
    	boolean tIsServerSide = isServerSide();
    	try {
	    	if (mTimer == 0) {
	    		markDirty();
	    		onTickFirst(tIsServerSide);
	    	}
	    	if (!isDead()) onTickStart(mTimer, tIsServerSide);
	    	if (!isDead()) super.updateEntity();
	    	if (!isDead()) onTick(mTimer, tIsServerSide);
	    	if (!isDead() && tIsServerSide && mTimer > 2 && (mSendClientData || onTickCheck(mTimer))) {
	    		sendClientData(mSendClientData, null);
	    		mSendClientData = F;
	    	}
	    	if (!isDead()) onTickResetChecks(mTimer, tIsServerSide);
	    	if (!isDead()) onTickEnd(mTimer, tIsServerSide);
    	} catch(Throwable e1) {
    		e1.printStackTrace(ERR);
    		setError((tIsServerSide?"Serverside: ":"Clientside: ") + e1);
    		try {
    			onTickFailed(mTimer, tIsServerSide);
    		} catch(Throwable e2) {
        		e2.printStackTrace(ERR);
        		setError((tIsServerSide?"Serverside: ":"Clientside: ") + e2);
    		}
    	}
    	mIsRunningTick = F;
    }
    
    /** Used to reset all Variables which have something to do with the detection of Changes. A super Call is important for this one! */
    public void onTickResetChecks(long aTimer, boolean aIsServerSide) {/**/}
    
    /** The very first Tick happening to this TileEntity */
    public void onTickFirst(boolean aIsServerSide) {/**/}
    
    /** The first Part of the Tick. */
    public void onTickStart(long aTimer, boolean aIsServerSide) {/**/}
    
    /** The regular Tick. */
    public void onTick(long aTimer, boolean aIsServerSide) {/**/}
    
    /** Use this to check if it is required to send an update to the Clients. If you want you can call "updateClientData", but then you need to return true in order for it to work.*/
    public boolean onTickCheck(long aTimer) {return F;}
    
    /** The absolutely last Part of the Tick. */
    public void onTickEnd(long aTimer, boolean aIsServerSide) {/**/}
    
    /** Gets called when there is an Exception happening during one of the Tick Functions. */
    public void onTickFailed(long aTimer, boolean aIsServerSide) {/**/}
}